#include "aliasdisplay.h"
#include <stdio.h>

InfoLabel::InfoLabel(QWidget * parent, const char * name)
  : QLabel( parent, name ){}

void InfoLabel::mouseDoubleClickEvent ( QMouseEvent * e)
{
  emit mouseDoubleClicked();
}

InfoDisplay::InfoDisplay(QWidget * parent, const char * name)
  : QScrollView( parent, name)
{
  QColorGroup cg(colorGroup());
  
  cg.setColor(QColorGroup::Foreground, Qt::cyan);
  cg.setColor(QColorGroup::Dark, Qt::black);
  cg.setColor(QColorGroup::Background, Qt::black);

  setPalette( QPalette(cg, cg, cg) );

  info = QString::null;
  
  infoLine = new InfoLabel( viewport() );
  tictoc = new QTimer;
  speed = 50;
  editing = false;
  
  setFrameStyle( NoFrame );
  infoLine->setFrameStyle( NoFrame );
  infoLine->setPalette( QPalette(cg, cg, cg) );
  setHScrollBarMode( AlwaysOff );
  setVScrollBarMode( AlwaysOff );
  addChild(infoLine);
  connect( tictoc, SIGNAL(timeout()),
	   this, SLOT(timeToScroll()) );

  connect( infoLine, SIGNAL(mouseDoubleClicked()),
	   this, SLOT(infoDoubleClickedSlot()) );

  infoUpdate("");
}

QString InfoDisplay::text()
{ return info; }

void InfoDisplay::infoUpdate(const char *str){
  QString temp;

  info = str;
  if(info.isEmpty()){
    tictoc->stop();
  }

  tictoc->start( speed );
  temp = info.simplifyWhiteSpace();
  temp.prepend("                                       ");
  temp.append ("                                       ");
  infoLine->setText(temp);
}

void InfoDisplay::setRunning(bool flag)
{
  if(flag){
    tictoc->start( speed  );
  }else
    tictoc->stop();
}

void InfoDisplay::setSpeed( int speed )
{
  this->speed = speed;
  if(tictoc->isActive())
    tictoc->start( speed );
}

void InfoDisplay::timeToScroll()
{
  int x;
  
  x = contentsX();
  scrollBy( 1 , 0 );

  if(x == contentsX())
    scrollBy( -1024, 0);

}


void InfoDisplay::infoDoubleClickedSlot() 
{
  editing = !editing;
  emit editingMode( editing );
}


AliasDisplay::AliasDisplay(QWidget * parent, const char * name)
  : QLineEdit(parent, name)
{
  QColorGroup cg(colorGroup());
  
  cg.setColor(QColorGroup::Text, Qt::cyan);
  cg.setColor(QColorGroup::Base, Qt::black);
  cg.setColor(QColorGroup::Background, Qt::black);

  setPalette( QPalette(cg, cg, cg) );

  bzero(alias, ALIASSIZE);
  setMaxLength( ALIASSIZE );
  setReadOnly( true );
  setFrame( false );
  setFocusPolicy( ClickFocus );
  connect( this, SIGNAL(returnPressed()), 
	   this, SLOT(returnPressedSlot()));
}


void AliasDisplay::aliasUpdate(const char *alias )
{
  strncpy( this->alias, alias, ALIASSIZE);
  setText(alias);
}

void AliasDisplay::mouseDoubleClickEvent ( QMouseEvent * e )
{
  setReadOnly( false );
}


void AliasDisplay::returnPressedSlot()
{
  QString str;
  QFocusData *focus;

  setReadOnly( true );
  if(edited()){
    strncpy(alias, (const char *)text(), ALIASSIZE);
    emit setAlias(alias);
  }
  
  focusNextPrevChild( false );
}
