#include <math.h>
#include <qpainter.h>

#include "meter.h"

#ifdef DEBUG_VER_
#include <stdio.h>
#endif // DEBUG_VER_

Meter::Meter( QWidget *parent, const char *name)
	: QWidget( parent, name )
{
	range.setRange( 0, 255 );
	range.setValue( 0 );
	range.setSteps( 1, 1 );

	ang_init = 0;
	len = 0;

	setBackgroundColor( Qt::black );
	setFocusPolicy( NoFocus );
	squelchOn = false;
}


void Meter::setValue( unsigned char val )
{
	range.setValue( val );
	repaint();
}

void Meter::setSquelchOn( bool val )
{
	squelchOn = val;
	repaint();
}

void Meter::paintEvent( QPaintEvent *e )
{
	QPainter p( this );
	float ratio;


	p.translate( width()/2, height() );

	if(squelchOn)
		p.setPen( Qt::green );
	else
		p.setPen( Qt::gray );

	p.drawText(-width()/4,-height()/2,width()/2, height()/4, 
				QPainter::AlignHCenter, "SIGNAL");

	p.setBrush( NoBrush );
	p.setPen( QPen( Qt::white, 2, QPainter::DashLine ) );
	p.drawEllipse( -len, -len+5, 2*len, 2*len );

	ratio = (double)(range.value() - range.minValue())/(range.maxValue()-range.minValue());
	p.rotate( ang_init + 180 + ratio*(180 - 2*ang_init) );

	p.setPen( QPen(Qt::yellow, 3) );
	p.drawLine( 0, 0, len,  0 );
}

void Meter::resizeEvent( QResizeEvent *e )
{
	len = e->size().height();
	if( 2*len > e->size().width() )
		len = (int)(e->size().width()*0.5774); 

	ang_init = (int)(acos( e->size().width()/2.0/len ) * 180.0 / M_PI);

#ifdef DEBUG_VER_
	fprintf( stderr, "Meter::resizeEvent - ang_init = %d\n", ang_init );
#endif // DEBUG_VER_ 
}
