#include "scan.h"

Scan::Scan()
{
  // set variables to default values
  beginFreq = DEFAULT_FREQ;
  endFreq = DEFAULT_FREQ;
  ts = DEFAULT_TS;
  freq = DEFAULT_FREQ;
  mode = DEFAULT_MODE;
  filter = DEFAULT_FILTER;
  sig = 0;
  prevSig = sig;
  msec = 500;
  tictoc.stop();

  connect( &tictoc, SIGNAL(timeout()), this, SLOT(timeToStepFreq())); 
}


void Scan::SetFrequency( pcrfreq_t beginFreq, pcrfreq_t endFreq)
{
  this->beginFreq = beginFreq;
  this->endFreq = endFreq;
  this->freq = beginFreq;
}

void Scan::SetTimeDelay( int msec )
{
  this->msec = msec;
}

void Scan::SetChannel( modulate_t mode, filter_t filter)
{
  this->mode = mode;
  this->filter = filter;

  // compute ts from filter
  switch( filter ){
   case FILTER_3kHz:  ts = 750; break;
   case FILTER_6kHz:  ts = 1500; break;
   case FILTER_15kHz: ts = 3750; break;
   case FILTER_50kHz: ts =12500; break;
   case FILTER_230kHz:ts =57500; break;   
  }
}


void Scan::sigStrengthUpdate( unsigned char sig)
{
  this->sig = sig;
  // sig change driven step algorithm
  //if(tictoc.isActive() && this->sig != sig){
  //   this->sig = sig;
  //   timeToStepFreq();
  //   tictoc.start( msec );  
  //}
}

void Scan::squelchOnUpdate( bool flag )
{
  if(!flag)
    this->sig = 0;
}


void Scan::Start()
{
  emit setMode( mode );
  emit setFilter( filter );
  emit setFreq( beginFreq );

  freq = beginFreq;
  sig = 0;
  prevSig = sig;
  tictoc.start( msec );
  emit scanBegun();
}

void Scan::Stop()
{
  tictoc.stop();
  emit scanEnded();
}


void Scan::timeToStepFreq()
{

  emit scanUpdate( freq , sig );

  prevSig = sig;
  freq += ts;

  if(freq >= endFreq){
    Stop();
    return;
  }

  emit setFreq( freq );
}
