#ifndef SCAN_DIALOG_H_
#define SCAN_DIALOG_H_

#include <qdialog.h>
#include <qlineedit.h>
#include <qcombobox.h>
#include <qcanvas.h>
#include "scan.h"
#include "pcrproto.h"

#define SignalLineID 1234

class SignalLine : public QCanvasLine
{
public:
  SignalLine( QCanvas * canvas );
  pcrfreq_t getFrequency();
  unsigned char getSig();
  void setFrequency(pcrfreq_t);
  void setSig(unsigned char);
  int rtti() const;

private:
  pcrfreq_t freq;
  unsigned char sig;
};


class ScanScreen : public QCanvasView
{
  Q_OBJECT
public:
  ScanScreen( QCanvas * viewing=0, QWidget * parent=0, const char * name=0, WFlags f=0);

  void clear();

signals:
  void setFreq( pcrfreq_t );

protected:
  void contentsMouseDoubleClickEvent ( QMouseEvent * );
};

class ScanDialog : public QDialog
{
  Q_OBJECT
public:
  ScanDialog( QWidget * parent=0, const char * name=0);
  ~ScanDialog();

// must be connected to PcrProxy
signals:
  void setFreq( pcrfreq_t );
  void setMode( modulate_t );
  void setFilter( filter_t );

public slots:
  void sigStrengthUpdate( unsigned char );
  void squelchOnUpdate( bool );

private slots:
  void StartSlot();
  void StopSlot();
  void PauseSlot();

  void scanBegun();
  void scanUpdate( pcrfreq_t, unsigned char );
  void scanEnded();
  
private:
  Scan        *scan;
  QPushButton *start;
  QPushButton *stop;
  QPushButton *pause;
  QLineEdit   *startFreqEdit;
  QLineEdit   *stopFreqEdit;
  QComboBox   *mode;
  QComboBox   *filter;
  ScanScreen  *screen; 
  QCanvas     *canvas;
  int         x;
  pcrfreq_t   freq, beginFreq, endFreq;
};
#endif // SCAN_DIALOG_H_
